use std::io::{self, BufRead};

enum Opcode {
    Add,
    Multiply,
    Halt,
}

impl From<u64> for Opcode {
    fn from(i: u64) -> Self {
        match i {
            1 => Opcode::Add,
            2 => Opcode::Multiply,
            99 => Opcode::Halt,
            _ => panic!("invalid opcode {}", i),
        }
    }
}

fn main() -> io::Result<()> {
    let stdin = io::stdin();
    let mut program: Vec<u64> = stdin
        .lock()
        .split(b',')
        .map(|o| {
            o.expect("")
                .iter()
                .filter(|i| **i >= b'0' && **i <= b'9')
                .fold(0, |s, i| (i - b'0' + 10 * s))
        })
        .map(|i| i as u64)
        .collect();
    program[1] = 12;
    program[2] = 2;
    let result = run(&mut program.clone());
    println!("result: {}", result);

    for a in 0..99 {
        for b in 0..99 {
            program[1] = a;
            program[2] = b;
            let result = run(&mut program.clone());
            if result == 19690720 {
                println!("a: {}, b: {}", a, b);
                return Ok(());
            }
        }
    }
    Ok(())
}

fn run(program: &mut Vec<u64>) -> u64 {
    let mut instruction_pointer: usize = 0;
    loop {
        let instruction = program[instruction_pointer];
        let advance = match instruction.into() {
            Opcode::Add => add(program, instruction_pointer),
            Opcode::Multiply => multiply(program, instruction_pointer),
            Opcode::Halt => return program[0],
        };
        instruction_pointer += advance;
    }
}

fn add(program: &mut Vec<u64>, pos: usize) -> usize {
    let a_pos = program[pos + 1] as usize;
    let b_pos = program[pos + 2] as usize;
    let c_pos = program[pos + 3] as usize;
    program[c_pos] = program[a_pos] + program[b_pos];
    4
}

fn multiply(program: &mut Vec<u64>, pos: usize) -> usize {
    let a_pos = program[pos + 1] as usize;
    let b_pos = program[pos + 2] as usize;
    let c_pos = program[pos + 3] as usize;
    program[c_pos] = program[a_pos] * program[b_pos];
    4
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn example1() {
        let mut program: Vec<u64> = vec![1, 9, 10, 3, 2, 3, 11, 0, 99, 30, 40, 50];
        assert_eq!(run(&mut program), 3500)
    }

    #[test]
    fn example2() {
        let mut program: Vec<u64> = vec![1, 0, 0, 0, 99];
        assert_eq!(run(&mut program), 2)
    }

    #[test]
    fn example3() {
        let mut program: Vec<u64> = vec![1, 1, 1, 4, 99, 5, 6, 0, 99];
        assert_eq!(run(&mut program), 30)
    }
}
